<?php


namespace FiloBlu\Imagick\Magento\Framework\Image\Adapter;


class ImageMagick extends \Magento\Framework\Image\Adapter\ImageMagick {

    protected $_helperImagick;
    
    public function __construct(
        \Magento\Framework\Filesystem $filesystem,
        \Psr\Log\LoggerInterface $logger,
        \FiloBlu\Imagick\Helper\Data $helper,
        array $data = []
    ) {
        $this->helperImagick = $helper;
        parent::__construct($filesystem,$logger, $data);
    }
    
     /**
     * Get/set quality, values in percentage from 0 to 100
     *
     * @param int $value
     * @return int
     */
    public function quality($value = null)
    {
        
        if (null !== $value) {
            $this->_quality = (int)$value;
        }
        
        if ($quality = $this->helperImagick->getStoreConfig("dev/image/force_image_quality")) {
            $this->_quality = $quality;
        }
        
        return $this->_quality;
    }
    
    /**
     * Change the image size
     *
     * @param null|int $frameWidth
     * @param null|int $frameHeight
     * @return void
     */
    public function resize($frameWidth = null, $frameHeight = null)
    {
        $format = $this->_imageHandler->getImageFormat();

        if ($format == 'GIF') {
            return;
        }
        
        $this->_checkCanProcess();
        $dims = $this->_adaptResizeValues($frameWidth, $frameHeight);

        $newImage = new \Imagick();
        $newImage->newImage(
            $dims['frame']['width'],
            $dims['frame']['height'],
            $this->_imageHandler->getImageBackgroundColor()
        );

        $this->_imageHandler->resizeImage(
            $dims['dst']['width'],
            $dims['dst']['height'],
            $this->helperImagick->getStoreConfig("dev/image/filter"),
            $this->helperImagick->getStoreConfig("dev/image/blur_factor"),
            ($this->helperImagick->getStoreConfig("dev/image/best_fit") ? true : false)
        );

        if ($this->_imageHandler->getImageWidth() < $this->_options['small_image']['width'] ||
            $this->_imageHandler->getImageHeight() < $this->_options['small_image']['height']
        ) {
            $this->_imageHandler->sharpenImage(
                $this->_options['sharpen']['radius'],
                $this->_options['sharpen']['deviation']
            );
        }

        $newImage->compositeImage(
            $this->_imageHandler,
            \Imagick::COMPOSITE_OVER,
            $dims['dst']['x'],
            $dims['dst']['y']
        );

        $newImage->setImageFormat($this->_imageHandler->getImageFormat());
        $this->_imageHandler->clear();
        $this->_imageHandler->destroy();
        $this->_imageHandler = $newImage;

        $this->refreshImageDimensions();
    }
    
    /**
     * Save image to specific path.
     * If some folders of path does not exist they will be created
     *
     * @param null|string $destination
     * @param null|string $newName
     * @return void
     * @throws \Exception  If destination path is not writable
     */
    public function save($destination = null, $newName = null)
    {
        $fileName = $this->_prepareDestination($destination, $newName);

         $format = $this->_imageHandler->getImageFormat();

        if ($format == 'GIF') {
            copy($this->_fileName, $fileName);
        }
        else {
            $this->_applyOptions();
            $this->_imageHandler->stripImage();
            $this->_imageHandler->writeImage($fileName);
        }
    }

}
